/*===========================================================================
  Copyright (C) 1995 European Southern Observatory (ESO)
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
  MA 02139, USA.
 
  Corresponding concerning ESO-MIDAS should be addressed as follows:
        Internet e-mail: midas@eso.org
        Postal address: European Southern Observatory
                        Data Management Division 
                        Karl-Schwarzschild-Strasse 2
                        D 85748 Garching bei Muenchen 
                        GERMANY
===========================================================================*/

#include <stdio.h>
#include <time.h> 

#include "mutil.h"

void siftDownInt(int numbers[], int rank[], int root, int bottom);
void siftDownFloat(float numbers[], int rank[], int root, int bottom);
void siftDownDouble(double numbers[], int rank[], int root, int bottom);

long int GetTickCount(); 
double   GetDuration(long int t1, long int t2); 

/* Validation code */ 
/* 
#define NUM_ITEMS 10

float flt_numbers[NUM_ITEMS];
int   int_numbers[NUM_ITEMS];
int   rank[NUM_ITEMS];

int main(void)
{
	int    i;
	long   t, tt;
        double dur;

	//seed random number generator
	srand(GetTickCount());

	//fill array with random integers
        heapFillRank(NUM_ITEMS, rank, 0); 
	for (i = 0; i < NUM_ITEMS; i++) {
		int_numbers[i] = rand()%NUM_ITEMS;
                flt_numbers[i] = (float)rand()/1000.; 
	}

	printf("Starting with Integer Heap sort.\n"); 
	for (i = 0; i < NUM_ITEMS; i++)
	  printf("[%d]=%i,%f ", rank[i], int_numbers[i], flt_numbers[i]);
        printf("\n"); 

	//perform heap sort on integer array
	tt = GetTickCount();
	heapSortInt(NUM_ITEMS, int_numbers, rank);
	t = GetTickCount();
        dur = GetDuration(tt, t); 

	printf("Done with Integer Heap sort. %d items %lf sec\n",NUM_ITEMS,dur);
	for (i = 0; i < NUM_ITEMS; i++)
	  printf("[%d]=%i ", rank[i],int_numbers[i]);
        printf("\n"); 

	printf("Starting with Float Heap sort.\n"); 
	//perform heap sort on float array
	tt = GetTickCount();
	heapSortFloat(NUM_ITEMS, flt_numbers, rank);
	t = GetTickCount();
        dur = GetDuration(tt, t); 

	printf("Done with Float Heap sort. %d items %lf sec\n",NUM_ITEMS,dur);

	for (i = 0; i < NUM_ITEMS; i++)
	  printf("[%d]=%f ", rank[i],flt_numbers[i]);
        printf("\n"); 

	return 0;

}
*/ 

void heapSortInt(int array_size, int numbers[], int rank[])
{
        int i;
        int temp; // Change to support type
        int tmp_rank; 

	for (i = (array_size / 2)-1; i >= 0; i--)
	        siftDownInt(numbers, rank, i, array_size);

	for (i = array_size-1; i >= 1; i--)
	{
		temp     = numbers[0];
                tmp_rank = rank[0]; 
		numbers[0] = numbers[i];
                rank[0]    = rank[i];
		numbers[i] = temp;
                rank[i]    = tmp_rank;
		siftDownInt(numbers, rank, 0, i-1);
	}
}

void siftDownInt(int numbers[], int rank[], int root, int bottom)
{
        int done, maxChild; 
        int temp; // Change to support type
        int tmp_rank; 

	done = 0;
	while ((root*2 <= bottom) && (!done))
	{
		if (root*2 == bottom)
			maxChild = root * 2;
		else if (numbers[root * 2] > numbers[root * 2 + 1])
			maxChild = root * 2;
		else
		    maxChild = root * 2 + 1;

		if (numbers[root] < numbers[maxChild])
		{
			temp     = numbers[root];
                        tmp_rank = rank[root];
			numbers[root] = numbers[maxChild];
                        rank[root]    = rank[maxChild]; 
			numbers[maxChild] = temp;
                        rank[maxChild]    = tmp_rank; 
			root = maxChild;
		}
		else
			done = 1;
	}
}

void heapSortFloat(int array_size, float numbers[], int rank[])
{
        int i;
        float temp; // Change to support type
        int tmp_rank; 

	for (i = (array_size / 2)-1; i >= 0; i--)
	        siftDownFloat(numbers, rank, i, array_size);

	for (i = array_size-1; i >= 1; i--)
	{
		temp     = numbers[0];
                tmp_rank = rank[0]; 
		numbers[0] = numbers[i];
                rank[0]    = rank[i];
		numbers[i] = temp;
                rank[i]    = tmp_rank;
		siftDownFloat(numbers, rank, 0, i-1);
	}
}

void siftDownFloat(float numbers[], int rank[], int root, int bottom)
{
        int   done, maxChild; 
        float temp; // Change to support type
        int   tmp_rank; 

	done = 0;
	while ((root*2 <= bottom) && (!done))
	{
		if (root*2 == bottom)
			maxChild = root * 2;
		else if (numbers[root * 2] > numbers[root * 2 + 1])
			maxChild = root * 2;
		else
		    maxChild = root * 2 + 1;

		if (numbers[root] < numbers[maxChild])
		{
			temp     = numbers[root];
                        tmp_rank = rank[root];
			numbers[root] = numbers[maxChild];
                        rank[root]    = rank[maxChild]; 
			numbers[maxChild] = temp;
                        rank[maxChild]    = tmp_rank; 
			root = maxChild;
		}
		else
			done = 1;
	}
}

void heapSortDouble(int array_size, double numbers[], int rank[])
{
        int i;
        double temp; // Change to support type
        int tmp_rank; 

	for (i = (array_size / 2)-1; i >= 0; i--)
	        siftDownDouble(numbers, rank, i, array_size);

	for (i = array_size-1; i >= 1; i--)
	{
		temp     = numbers[0];
                tmp_rank = rank[0]; 
		numbers[0] = numbers[i];
                rank[0]    = rank[i];
		numbers[i] = temp;
                rank[i]    = tmp_rank;
		siftDownDouble(numbers, rank, 0, i-1);
	}
}

void siftDownDouble(double numbers[], int rank[], int root, int bottom)
{
        int    done, maxChild; 
        double temp; // Change to support type
        int    tmp_rank; 

	done = 0;
	while ((root*2 <= bottom) && (!done))
	{
		if (root*2 == bottom)
			maxChild = root * 2;
		else if (numbers[root * 2] > numbers[root * 2 + 1])
			maxChild = root * 2;
		else
		    maxChild = root * 2 + 1;

		if (numbers[root] < numbers[maxChild])
		{
			temp     = numbers[root];
                        tmp_rank = rank[root];
			numbers[root] = numbers[maxChild];
                        rank[root]    = rank[maxChild]; 
			numbers[maxChild] = temp;
                        rank[maxChild]    = tmp_rank; 
			root = maxChild;
		}
		else
			done = 1;
	}
}

void  heapFillRank(int array_size, int rank[], int offset) 
{
  int i; 
  for (i=0; i<array_size; i++) 
       rank[i] = i + offset; 
}

long int GetTickCount() { 
    clock_t t2 = clock();
    return((long int) t2); 
}

double GetDuration(long int t1, long int t2) { 
    double dur = 1000.0*(t2-t1)/CLOCKS_PER_SEC;
    return(dur); 
}
